unit rusnuupc;

{
rusnuupc - support for using RusNews with UUPC/Extended.

- posting support for UUPC. The rnews module in uupc is flawed;
  only incoming news is handled. So posting needs to be done by
  using uucp to queue the D. and X. files, after they are crea-
  ted by this module.

By Henrik Storner (storner@osiris.ping.dk)

}

interface

uses rusnglob, rusnproc;

procedure uupc_post(articlefn: string; newsgroups: string);



implementation

var
  seqfile, tmp: text;
  seq: integer; seqstr: string;
  i: integer;
  d_name, x_name, short_x_name: string;
  uucp_args: string;
  ngdir: string;

procedure uupc_post;

const
  hexchars : string[16] = '0123456789abcdef';

begin

{ Get a sequence number for the D. and X. files }
assign(seqfile, userdir+'\nseq');
{$I-} 
reset(seqfile);
if ioresult<>0 then
   begin
   { Sequence file not found }
   seq:= 0;
   end
else
   begin
   readln(seqfile, seq);
   if ioresult<>0 then seq:= 0;
   close(seqfile);
   end;
seq:= seq+1;
rewrite(seqfile); 
if ioresult=0 then 
   writeln(seqfile, seq)
else
   { Do some error handling }
   ;
close(seqfile);
{$I+}

{ Get it in 4-digit hex }
seqstr:= '';
for i:= 1 to 4 do
   begin
   seqstr:= hexchars[1+(seq mod 16)] + seqstr;
   seq:= seq div 16;
   end;

d_name:= 'D.' + newsname + seqstr;
x_name:= 'X.' + newsname + seqstr;
short_x_name:= temporarydir + '\X_' + seqstr + '.' + copy(newsname, 1, 3);

assign(tmp, short_x_name); rewrite(tmp); 
writeln(tmp, 'U news ', newsname);
writeln(tmp, 'Z');
writeln(tmp, 'F ', d_name);
writeln(tmp, 'I ', d_name);
writeln(tmp, 'C rnews');
close(tmp);

uucp_args:= '-C ' + articlefn + ' ' + smarthost + '!' + d_name;
execp('uucp.exe', uucp_args);
uucp_args:= '-C ' + short_x_name + ' ' + smarthost + '!' + x_name;
execp('uucp.exe', uucp_args);

assign(tmp, short_x_name); erase(tmp);

{ Post the article locally. For this, rnews works! }
uucp_args:= '-f ' + articlefn;
execp('rnews.exe', uucp_args);

end;



end { Module ends }.
